 /* Copyright 2024 by COG9 Enterprises LLC
 * All rights reserved.
 * Title: ocmdline.c
 * Author: Jed Marti
 * Description: Parse command line options.
 * Created: Tue Dec  3 08:12:03 2024
 * Revision History:
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>

#include "meshmail.h"

// Noise level
uint8_t Noise = 0;

// AREDN port number. Must be same as MQTT.
int32_t AREDNPortnum = DEFAULT_AREDN;

// Mail file directory.
char *MailDir = DEFAULT_MAIL_DIR;


static int32_t nextarg(int32_t i, int32_t argc, const char *emsg)
/*     Move to the next command line value and signal error if we fall off the end.
 * Parameters:
 *  i: The current index.
 *  argc: From OS, the number of items on the command line.
 *  emsg: Item we're looking for. Used in error message.
 * External References: none
 * Returned values: i + 1.
 * Errors detected: Exceeding the parameter count.
 */
{
  if (++i >= argc)
  {
    fprintf(stderr, "Looking for %s argument but not found\n", emsg);
    exit(-1);
  }
  return i;
}



void cmdline(int32_t argc, char *argv[])
/*     Scan the OS command line to change default values from meshmail.h.
 * Parameters:
 *  argc: unaltered from OS startup, count to examine + 1 (program name)
 *  argv[]: A vector of strings from the command lin e.
 * External References:
 *   Noise: Noise level. 0 (default), not a peep, 5 and greater to the point of obnoxiousness.
 *   AREDNPortnum: The TCP port number to listen on for messages from the MQTT server.
 *   MailDir: The directory to store e-mail from the MQTT server.
 * Returned values: none, values in global variables changed.
 * Errors detected: port number too low signals warning about being root. Missing parameter
 *   value, unknown option.
 */
{
  int32_t i;

  for (i = 1; i < argc; ++i)
  {
    // Noise level. 0=quiet, 5 = everything.
    if (strcasecmp("-noise", argv[i]) == 0 ||
        strcasecmp("-n", argv[i]) == 0)
    {
      i = nextarg(i, argc, "-n/-noise integer");
      Noise = (uint8_t) atoi(argv[i]);
    }

    // Port number to listen on for mail messages.
    else if ((strcasecmp("-p", argv[i]) == 0) ||
             (strcasecmp("-portnum", argv[i]) == 0))
    {
      i = nextarg(i, argc, "-p/-portnum");
      AREDNPortnum = atoi(argv[i]);
      if (AREDNPortnum < 1024)
        fprintf(stderr, "Portnumber %d < 1024 requires root access\n", AREDNPortnum);
    }

    // The directory to store mail in.
    else if (strcasecmp("-md", argv[i]) == 0 ||
             strcasecmp("-maildir", argv[i]) == 0)
    {
      i = nextarg(i, argc, "-md/-maildir");
      MailDir = argv[i];
    }
         
    // Show the options.
    else if (strcasecmp("-help", argv[i]) == 0 ||
             strcasecmp("-h", argv[i]) == 0)
    {
      fprintf(stderr, "\
Options:\n\
  -md -maildir       string    Where messages are stored (%s).\n\
  -n  -noise         integer   Set the output noise level (%d) 0-5\n\
  -p  -portnum       integer   AREDN port number %d.\n",
              MailDir,
              Noise,
              AREDNPortnum);
      exit(0);
    }

    // Halt. What is user trying to do?
    else
    {
      fprintf(stderr, "%s:%s:%d Option %s unknown\n", __FILE__, __func__, __LINE__, argv[i]);
      exit(-1);
    }
  }
}


// EOF
